@@ -36,7 +36,6 @@
 #include "config.h"
 #endif
 
-#include "gstglelements.h"
 #include "gstgleffects.h"
 
 #define GST_CAT_DEFAULT gst_gl_effects_debug
@@ -57,15 +56,6 @@ enum
 G_DEFINE_TYPE_WITH_CODE (GstGLEffects, gst_gl_effects, GST_TYPE_GL_FILTER,
     DEBUG_INIT);
 
-static gboolean
-gst_element_init_gleffects (GstPlugin * plugin)
-{
-  gl_element_init (plugin);
-  return gst_gl_effects_register_filters (plugin, GST_RANK_NONE);
-}
-
-GST_ELEMENT_REGISTER_DEFINE_CUSTOM (gleffects, gst_element_init_gleffects);
-
 static void gst_gl_effects_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_gl_effects_get_property (GObject * object, guint prop_id,
@@ -349,7 +339,7 @@ gst_gl_effects_class_init (GstGLEffectsClass * klass)
 
   klass->filter_descriptor = NULL;
 
-  gst_element_class_set_static_metadata (element_class,
+  gst_element_class_set_metadata (element_class,
       "Gstreamer OpenGL Effects", "Filter/Effect/Video",
       "GL Shading Language effects",
       "Filippo Argiolas <filippo.argiolas@gmail.com>");
@@ -660,7 +650,7 @@ gst_gl_effects_filters_descriptors (void)
 gboolean
 gst_gl_effects_register_filters (GstPlugin * plugin, GstRank rank)
 {
-  static gsize registered = 0;
+  static volatile gsize registered = 0;
 
   if (g_once_init_enter (&registered)) {
     GTypeInfo info = {
