@@ -210,6 +210,9 @@ gst_gl_stereo_mix_class_init (GstGLStereoMixClass * klass)
 
   base_mix_class->supported_gl_api =
       GST_GL_API_GLES2 | GST_GL_API_OPENGL | GST_GL_API_OPENGL3;
+
+  gst_type_mark_as_plugin_api (GST_TYPE_GL_STEREO_DOWNMIX, 0);
+  gst_type_mark_as_plugin_api (GST_TYPE_GL_STEREO_MIX_PAD, 0);
 }
 
 static void
@@ -363,7 +366,7 @@ gst_gl_stereo_mix_aggregate_frames (GstVideoAggregator * vagg,
   GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
   /* If we're operating in frame-by-frame mode, push
    * the primary view now, and let the parent class
-   * push the remaining auxilliary view */
+   * push the remaining auxiliary view */
   if (GST_VIDEO_INFO_MULTIVIEW_MODE (&vagg->info) ==
       GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME) {
     /* Transfer the timestamps video-agg put on the aux buffer */
@@ -700,6 +703,10 @@ gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer)
   else
     views = 1;
 
+  /* We can configure the view_converter now */
+  gst_gl_view_convert_set_context (mixer->viewconvert,
+      GST_GL_BASE_MIXER (mixer)->context);
+
   if (gst_gl_view_convert_submit_input_buffer (mixer->viewconvert,
           FALSE, inbuf) != GST_FLOW_OK)
     return FALSE;
