@@ -22,12 +22,13 @@
 
 /**
  * SECTION:element-theoraenc
+ * @title: theoraenc
  * @see_also: theoradec, oggmux
  *
  * This element encodes raw video into a Theora stream.
- * <ulink url="http://www.theora.org/">Theora</ulink> is a royalty-free
- * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
- * Foundation</ulink>, based on the VP3 codec.
+ * [Theora](http://www.theora.org/) is a royalty-free
+ * video codec maintained by the [Xiph.org Foundation](http://www.xiph.org/),
+ * based on the VP3 codec.
  *
  * The theora codec internally only supports encoding of images that are a
  * multiple of 16 pixels in both X and Y direction. It is however perfectly
@@ -36,8 +37,8 @@
  * set up a correct cropping region if the dimensions are not multiples of 16
  * pixels.
  *
- * To control the quality of the encoding, the #GstTheoraEnc::bitrate and
- * #GstTheoraEnc::quality properties can be used. These two properties are
+ * To control the quality of the encoding, the #GstTheoraEnc:bitrate and
+ * #GstTheoraEnc:quality properties can be used. These two properties are
  * mutualy exclusive. Setting the bitrate property will produce a constant
  * bitrate (CBR) stream while setting the quality property will produce a
  * variable bitrate (VBR) stream.
@@ -45,14 +46,14 @@
  * A videorate element is often required in front of theoraenc, especially
  * when transcoding and when putting Theora into the Ogg container.
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 -v videotestsrc num-buffers=500 ! video/x-raw,width=1280,height=720 ! queue ! progressreport ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
- * ]| This example pipeline will encode a test video source to theora muxed in an
+ * ]|
+ *  This example pipeline will encode a test video source to theora muxed in an
  * ogg container. Refer to the theoradec documentation to decode the create
  * stream.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -212,13 +213,12 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   gobject_class->get_property = theora_enc_get_property;
   gobject_class->finalize = theora_enc_finalize;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&theora_enc_src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&theora_enc_sink_factory));
-  gst_element_class_set_static_metadata (element_class,
-      "Theora video encoder", "Codec/Encoder/Video",
-      "encode raw YUV video to a theora stream",
+  gst_element_class_add_static_pad_template (element_class,
+      &theora_enc_src_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &theora_enc_sink_factory);
+  gst_element_class_set_static_metadata (element_class, "Theora video encoder",
+      "Codec/Encoder/Video", "encode raw YUV video to a theora stream",
       "Wim Taymans <wim@fluendo.com>");
 
   gstvideo_encoder_class->start = GST_DEBUG_FUNCPTR (theora_enc_start);
@@ -301,6 +301,8 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_MULTIPASS_MODE, 0);
 }
 
 static void
@@ -704,7 +706,7 @@ theora_enc_init_buffer (th_ycbcr_buffer buf, GstVideoFrame * frame)
   GstVideoInfo vinfo;
   guint i;
 
-  /* According to Theora developer Timothy Terriberry, the Theora 
+  /* According to Theora developer Timothy Terriberry, the Theora
    * encoder will not use memory outside of pic_width/height, even when
    * the frame size is bigger. The values outside this region will be encoded
    * to default values.
@@ -879,7 +881,7 @@ theora_enc_handle_frame (GstVideoEncoder * benc, GstVideoCodecFrame * frame)
   enc = GST_THEORA_ENC (benc);
 
   /* we keep track of two timelines.
-   * - The timestamps from the incomming buffers, which we copy to the outgoing
+   * - The timestamps from the incoming buffers, which we copy to the outgoing
    *   encoded buffers as-is. We need to do this as we simply forward the
    *   newsegment events.
    * - The running_time of the buffers, which we use to construct the granulepos
@@ -1199,10 +1201,3 @@ theora_enc_get_property (GObject * object, guint prop_id,
       break;
   }
 }
-
-gboolean
-gst_theora_enc_register (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "theoraenc",
-      GST_RANK_PRIMARY, GST_TYPE_THEORA_ENC);
-}
