@@ -146,6 +146,9 @@ struct _GstAppSrcPrivate
 
   GstCaps *last_caps;
   GstCaps *current_caps;
+  GstSegment last_segment;
+  GstSegment current_segment;
+  gboolean pending_custom_segment;
 
   gint64 size;
   GstClockTime duration;
@@ -166,6 +169,7 @@ struct _GstAppSrcPrivate
   guint64 max_latency;
   gboolean emit_signals;
   guint min_percent;
+  gboolean handle_segment_change;
 
   Callbacks *callbacks;
 };
@@ -201,6 +205,7 @@ enum
 #define DEFAULT_PROP_MIN_PERCENT   0
 #define DEFAULT_PROP_CURRENT_LEVEL_BYTES   0
 #define DEFAULT_PROP_DURATION      GST_CLOCK_TIME_NONE
+#define DEFAULT_PROP_HANDLE_SEGMENT_CHANGE FALSE
 
 enum
 {
@@ -218,6 +223,7 @@ enum
   PROP_MIN_PERCENT,
   PROP_CURRENT_LEVEL_BYTES,
   PROP_DURATION,
+  PROP_HANDLE_SEGMENT_CHANGE,
   PROP_LAST
 };
 
@@ -288,7 +294,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   gobject_class->get_property = gst_app_src_get_property;
 
   /**
-   * GstAppSrc::caps:
+   * GstAppSrc:caps:
    *
    * The GstCaps that will negotiated downstream and will be put
    * on outgoing buffers.
@@ -298,7 +304,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           "The allowed caps for the src pad", GST_TYPE_CAPS,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::format:
+   * GstAppSrc:format:
    *
    * The format to use for segment events. When the source is producing
    * timestamped buffers this property should be set to GST_FORMAT_TIME.
@@ -308,7 +314,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           "The format of the segment events and seek", GST_TYPE_FORMAT,
           DEFAULT_PROP_FORMAT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::size:
+   * GstAppSrc:size:
    *
    * The total size in bytes of the data stream. If the total size is known, it
    * is recommended to configure it with this property.
@@ -319,7 +325,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           -1, G_MAXINT64, DEFAULT_PROP_SIZE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::stream-type:
+   * GstAppSrc:stream-type:
    *
    * The type of stream that this source is producing.  For seekable streams the
    * application should connect to the seek-data signal.
@@ -330,7 +336,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           DEFAULT_PROP_STREAM_TYPE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::max-bytes:
+   * GstAppSrc:max-bytes:
    *
    * The maximum amount of bytes that can be queued internally.
    * After the maximum amount of bytes are queued, appsrc will emit the
@@ -342,7 +348,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           0, G_MAXUINT64, DEFAULT_PROP_MAX_BYTES,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::block:
+   * GstAppSrc:block:
    *
    * When max-bytes are queued and after the enough-data signal has been emitted,
    * block any further push-buffer calls until the amount of queued bytes drops
@@ -354,7 +360,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           DEFAULT_PROP_BLOCK, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::is-live:
+   * GstAppSrc:is-live:
    *
    * Instruct the source to behave like a live source. This includes that it
    * will only push out buffers in the PLAYING state.
@@ -364,7 +370,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           "Whether to act as a live source",
           DEFAULT_PROP_IS_LIVE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstAppSrc::min-latency:
+   * GstAppSrc:min-latency:
    *
    * The minimum latency of the source. A value of -1 will use the default
    * latency calculations of #GstBaseSrc.
@@ -387,7 +393,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::emit-signals:
+   * GstAppSrc:emit-signals:
    *
    * Make appsrc emit the "need-data", "enough-data" and "seek-data" signals.
    * This option is by default enabled for backwards compatibility reasons but
@@ -400,7 +406,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::min-percent:
+   * GstAppSrc:min-percent:
    *
    * Make appsrc emit the "need-data" signal when the amount of bytes in the
    * queue drops below this percentage of max-bytes.
@@ -412,7 +418,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::current-level-bytes:
+   * GstAppSrc:current-level-bytes:
    *
    * The number of currently queued bytes inside appsrc.
    *
@@ -425,7 +431,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::duration:
+   * GstAppSrc:duration:
    *
    * The total duration in nanoseconds of the data stream. If the total duration is known, it
    * is recommended to configure it with this property.
@@ -439,7 +445,30 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstAppSrc::need-data:
+   * GstAppSrc:handle-segment-change:
+   *
+   * When enabled, appsrc will check GstSegment in GstSample which was
+   * pushed via gst_app_src_push_sample() or "push-sample" signal action.
+   * If a GstSegment is changed, corresponding segment event will be followed
+   * by next data flow.
+   *
+   * FIXME: currently only GST_FORMAT_TIME format is supported and therefore
+   * GstAppSrc::format should be time. However, possibly #GstAppSrc can support
+   * other formats.
+   *
+   * Since: 1.18
+   */
+  g_object_class_install_property (gobject_class, PROP_HANDLE_SEGMENT_CHANGE,
+      g_param_spec_boolean ("handle-segment-change", "Handle Segment Change",
+          "Whether to detect and handle changed time format GstSegment in "
+          "GstSample. User should set valid GstSegment in GstSample. "
+          "Must set format property as \"time\" to enable this property",
+          DEFAULT_PROP_HANDLE_SEGMENT_CHANGE,
+          G_PARAM_READWRITE | GST_PARAM_MUTABLE_READY |
+          G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstAppSrc:need-data:
    * @appsrc: the appsrc element that emitted the signal
    * @length: the amount of bytes needed.
    *
@@ -458,7 +487,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
       NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_UINT);
 
   /**
-   * GstAppSrc::enough-data:
+   * GstAppSrc:enough-data:
    * @appsrc: the appsrc element that emitted the signal
    *
    * Signal that the source has enough data. It is recommended that the
@@ -471,7 +500,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
       NULL, NULL, NULL, G_TYPE_NONE, 0, G_TYPE_NONE);
 
   /**
-   * GstAppSrc::seek-data:
+   * GstAppSrc:seek-data:
    * @appsrc: the appsrc element that emitted the signal
    * @offset: the offset to seek to
    *
@@ -487,7 +516,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
       NULL, NULL, NULL, G_TYPE_BOOLEAN, 1, G_TYPE_UINT64);
 
    /**
-    * GstAppSrc::push-buffer:
+    * GstAppSrc:push-buffer:
     * @appsrc: the appsrc
     * @buffer: a buffer to push
     *
@@ -505,7 +534,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
       GST_TYPE_FLOW_RETURN, 1, GST_TYPE_BUFFER);
 
    /**
-    * GstAppSrc::push-buffer-list:
+    * GstAppSrc:push-buffer-list:
     * @appsrc: the appsrc
     * @buffer_list: a buffer list to push
     *
@@ -526,7 +555,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
       GST_TYPE_FLOW_RETURN, 1, GST_TYPE_BUFFER_LIST);
 
   /**
-    * GstAppSrc::push-sample:
+    * GstAppSrc:push-sample:
     * @appsrc: the appsrc
     * @sample: a sample from which extract buffer to push
     *
@@ -543,7 +572,6 @@ gst_app_src_class_init (GstAppSrcClass * klass)
     * becomes available in the queue.
     *
     * Since: 1.6
-    *
     */
   gst_app_src_signals[SIGNAL_PUSH_SAMPLE] =
       g_signal_new ("push-sample", G_TYPE_FROM_CLASS (klass),
@@ -553,7 +581,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
 
 
    /**
-    * GstAppSrc::end-of-stream:
+    * GstAppSrc:end-of-stream:
     * @appsrc: the appsrc
     *
     * Notify @appsrc that no more buffer are available.
@@ -614,6 +642,7 @@ gst_app_src_init (GstAppSrc * appsrc)
   priv->max_latency = DEFAULT_PROP_MAX_LATENCY;
   priv->emit_signals = DEFAULT_PROP_EMIT_SIGNALS;
   priv->min_percent = DEFAULT_PROP_MIN_PERCENT;
+  priv->handle_segment_change = DEFAULT_PROP_HANDLE_SEGMENT_CHANGE;
 
   gst_base_src_set_live (GST_BASE_SRC (appsrc), DEFAULT_PROP_IS_LIVE);
 }
@@ -760,6 +789,9 @@ gst_app_src_set_property (GObject * object, guint prop_id,
     case PROP_DURATION:
       gst_app_src_set_duration (appsrc, g_value_get_uint64 (value));
       break;
+    case PROP_HANDLE_SEGMENT_CHANGE:
+      priv->handle_segment_change = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -823,6 +855,9 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_DURATION:
       g_value_set_uint64 (value, gst_app_src_get_duration (appsrc));
       break;
+    case PROP_HANDLE_SEGMENT_CHANGE:
+      g_value_set_boolean (value, priv->handle_segment_change);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -895,6 +930,9 @@ gst_app_src_start (GstBaseSrc * bsrc)
   g_mutex_unlock (&priv->mutex);
 
   gst_base_src_set_format (bsrc, priv->format);
+  gst_segment_init (&priv->last_segment, priv->format);
+  gst_segment_init (&priv->current_segment, priv->format);
+  priv->pending_custom_segment = FALSE;
 
   return TRUE;
 }
@@ -1056,6 +1094,9 @@ gst_app_src_do_seek (GstBaseSrc * src, GstSegment * segment)
     GST_DEBUG_OBJECT (appsrc, "flushing queue");
     g_mutex_lock (&priv->mutex);
     gst_app_src_flush_queued (appsrc, TRUE);
+    gst_segment_copy_into (segment, &priv->last_segment);
+    gst_segment_copy_into (segment, &priv->current_segment);
+    priv->pending_custom_segment = FALSE;
     g_mutex_unlock (&priv->mutex);
     priv->is_eos = FALSE;
   } else {
@@ -1258,11 +1299,9 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
         *buf = GST_BUFFER (obj);
         buf_size = gst_buffer_get_size (*buf);
         GST_LOG_OBJECT (appsrc, "have buffer %p of size %u", *buf, buf_size);
-      } else {
+      } else if (GST_IS_BUFFER_LIST (obj)) {
         GstBufferList *buffer_list;
 
-        g_assert (GST_IS_BUFFER_LIST (obj));
-
         buffer_list = GST_BUFFER_LIST (obj);
 
         buf_size = gst_buffer_list_calculate_size (buffer_list);
@@ -1272,6 +1311,29 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
 
         gst_base_src_submit_buffer_list (bsrc, buffer_list);
         *buf = NULL;
+      } else if (GST_IS_EVENT (obj)) {
+        GstEvent *event = GST_EVENT (obj);
+        const GstSegment *segment = NULL;
+
+        gst_event_parse_segment (event, &segment);
+        g_assert (segment != NULL);
+
+        if (!gst_segment_is_equal (&priv->current_segment, segment)) {
+          GST_DEBUG_OBJECT (appsrc,
+              "Update new segment %" GST_PTR_FORMAT, event);
+          if (!gst_base_src_new_segment (bsrc, segment)) {
+            GST_ERROR_OBJECT (appsrc,
+                "Couldn't set new segment %" GST_PTR_FORMAT, event);
+            gst_event_unref (event);
+            goto invalid_segment;
+          }
+          gst_segment_copy_into (segment, &priv->current_segment);
+        }
+
+        gst_event_unref (event);
+        continue;
+      } else {
+        g_assert_not_reached ();
       }
 
       priv->queued_bytes -= buf_size;
@@ -1343,6 +1405,14 @@ seek_error:
         GST_ERROR_SYSTEM);
     return GST_FLOW_ERROR;
   }
+
+invalid_segment:
+  {
+    g_mutex_unlock (&priv->mutex);
+    GST_ELEMENT_ERROR (appsrc, LIBRARY, SETTINGS,
+        (NULL), ("Failed to configure the provided input segment."));
+    return GST_FLOW_ERROR;
+  }
 }
 
 /* external API */
@@ -1897,6 +1967,14 @@ gst_app_src_push_internal (GstAppSrc * appsrc, GstBuffer * buffer,
       break;
   }
 
+  if (priv->pending_custom_segment) {
+    GstEvent *event = gst_event_new_segment (&priv->last_segment);
+
+    GST_DEBUG_OBJECT (appsrc, "enqueue new segment %" GST_PTR_FORMAT, event);
+    gst_queue_array_push_tail (priv->queue, event);
+    priv->pending_custom_segment = FALSE;
+  }
+
   if (buflist != NULL) {
     GST_DEBUG_OBJECT (appsrc, "queueing buffer list %p", buflist);
     if (!steal_ref)
@@ -1947,6 +2025,7 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
 static GstFlowReturn
 gst_app_src_push_sample_internal (GstAppSrc * appsrc, GstSample * sample)
 {
+  GstAppSrcPrivate *priv = appsrc->priv;
   GstBufferList *buffer_list;
   GstBuffer *buffer;
   GstCaps *caps;
@@ -1960,6 +2039,30 @@ gst_app_src_push_sample_internal (GstAppSrc * appsrc, GstSample * sample)
     GST_WARNING_OBJECT (appsrc, "received sample without caps");
   }
 
+  if (priv->handle_segment_change && priv->format == GST_FORMAT_TIME) {
+    GstSegment *segment = gst_sample_get_segment (sample);
+
+    if (segment->format != GST_FORMAT_TIME) {
+      GST_LOG_OBJECT (appsrc, "format %s is not supported",
+          gst_format_get_name (segment->format));
+      goto handle_buffer;
+    }
+
+    g_mutex_lock (&priv->mutex);
+    if (gst_segment_is_equal (&priv->last_segment, segment)) {
+      GST_LOG_OBJECT (appsrc, "segment wasn't changed");
+      g_mutex_unlock (&priv->mutex);
+      goto handle_buffer;
+    }
+
+    /* will be pushed to queue with next buffer/buffer-list */
+    gst_segment_copy_into (segment, &priv->last_segment);
+    priv->pending_custom_segment = TRUE;
+    g_mutex_unlock (&priv->mutex);
+  }
+
+handle_buffer:
+
   buffer = gst_sample_get_buffer (sample);
   if (buffer != NULL)
     return gst_app_src_push_buffer_full (appsrc, buffer, FALSE);
