@@ -47,6 +47,10 @@ GST_DEBUG_CATEGORY_STATIC (gst_video_filter_debug);
 G_DEFINE_ABSTRACT_TYPE (GstVideoFilter, gst_video_filter,
     GST_TYPE_BASE_TRANSFORM);
 
+/* cached quark to avoid contention on the global quark table lock */
+#define META_TAG_VIDEO meta_tag_video_quark
+static GQuark meta_tag_video_quark;
+
 /* Answer the allocation query downstream. */
 static gboolean
 gst_video_filter_propose_allocation (GstBaseTransform * trans,
@@ -128,6 +132,9 @@ gst_video_filter_decide_allocation (GstBaseTransform * trans, GstQuery * query)
   if (gst_query_get_n_allocation_pools (query) > 0) {
     gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
 
+    if (!pool)
+      gst_query_parse_allocation (query, &outcaps, NULL);
+
     update_pool = TRUE;
   } else {
     GstVideoInfo vinfo;
@@ -262,9 +269,10 @@ gst_video_filter_transform (GstBaseTransform * trans, GstBuffer * inbuf,
       goto invalid_buffer;
 
     if (!gst_video_frame_map (&out_frame, &filter->out_info, outbuf,
-            GST_MAP_WRITE | GST_VIDEO_FRAME_MAP_FLAG_NO_REF))
+            GST_MAP_WRITE | GST_VIDEO_FRAME_MAP_FLAG_NO_REF)) {
+      gst_video_frame_unmap (&in_frame);
       goto invalid_buffer;
-
+    }
     res = fclass->transform_frame (filter, &in_frame, &out_frame);
 
     gst_video_frame_unmap (&out_frame);
@@ -349,8 +357,7 @@ gst_video_filter_transform_meta (GstBaseTransform * trans, GstBuffer * inbuf,
   tags = gst_meta_api_type_get_tags (info->api);
 
   if (!tags || (g_strv_length ((gchar **) tags) == 1
-          && gst_meta_api_type_has_tag (info->api,
-              g_quark_from_string (GST_META_TAG_VIDEO_STR))))
+          && gst_meta_api_type_has_tag (info->api, META_TAG_VIDEO)))
     return TRUE;
 
   return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (trans, inbuf,
@@ -382,6 +389,8 @@ gst_video_filter_class_init (GstVideoFilterClass * g_class)
 
   GST_DEBUG_CATEGORY_INIT (gst_video_filter_debug, "videofilter", 0,
       "videofilter");
+
+  meta_tag_video_quark = g_quark_from_static_string (GST_META_TAG_VIDEO_STR);
 }
 
 static void
