@@ -40,7 +40,7 @@
  * its output while playing.
  *
  * If you want to control the manner in which incoming data gets converted,
- * see the #GstAudioAggregatorPad:converter-config property, which will let
+ * see the #GstAudioAggregatorConvertPad:converter-config property, which will let
  * you for example change the way in which channels may get remapped.
  *
  * The input pads are from a GstPad subclass and have additional
@@ -224,8 +224,8 @@ gst_audiomixer_class_init (GstAudioMixerClass * klass)
   GstElementClass *gstelement_class = (GstElementClass *) klass;
   GstAudioAggregatorClass *aagg_class = (GstAudioAggregatorClass *) klass;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_audiomixer_src_template);
+  gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
+      &gst_audiomixer_src_template, GST_TYPE_AUDIO_AGGREGATOR_CONVERT_PAD);
   gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
       &gst_audiomixer_sink_template, GST_TYPE_AUDIO_MIXER_PAD);
   gst_element_class_set_static_metadata (gstelement_class, "AudioMixer",
@@ -238,6 +238,8 @@ gst_audiomixer_class_init (GstAudioMixerClass * klass)
       GST_DEBUG_FUNCPTR (gst_audiomixer_release_pad);
 
   aagg_class->aggregate_one_buffer = gst_audiomixer_aggregate_one_buffer;
+
+  gst_type_mark_as_plugin_api (GST_TYPE_AUDIO_MIXER_PAD, 0);
 }
 
 static void
@@ -295,6 +297,8 @@ gst_audiomixer_aggregate_one_buffer (GstAudioAggregator * aagg,
   GstMapInfo inmap;
   GstMapInfo outmap;
   gint bpf;
+  GstAggregator *agg = GST_AGGREGATOR (aagg);
+  GstAudioAggregatorPad *srcpad = GST_AUDIO_AGGREGATOR_PAD (agg->srcpad);
 
   GST_OBJECT_LOCK (aagg);
   GST_OBJECT_LOCK (aaggpad);
@@ -306,7 +310,7 @@ gst_audiomixer_aggregate_one_buffer (GstAudioAggregator * aagg,
     return FALSE;
   }
 
-  bpf = GST_AUDIO_INFO_BPF (&aagg->info);
+  bpf = GST_AUDIO_INFO_BPF (&srcpad->info);
 
   gst_buffer_map (outbuf, &outmap, GST_MAP_READWRITE);
   gst_buffer_map (inbuf, &inmap, GST_MAP_READ);
@@ -315,92 +319,92 @@ gst_audiomixer_aggregate_one_buffer (GstAudioAggregator * aagg,
 
   /* further buffers, need to add them */
   if (pad->volume == 1.0) {
-    switch (aagg->info.finfo->format) {
+    switch (srcpad->info.finfo->format) {
       case GST_AUDIO_FORMAT_U8:
         audiomixer_orc_add_u8 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S8:
         audiomixer_orc_add_s8 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_U16:
         audiomixer_orc_add_u16 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S16:
         audiomixer_orc_add_s16 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_U32:
         audiomixer_orc_add_u32 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S32:
         audiomixer_orc_add_s32 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_F32:
         audiomixer_orc_add_f32 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_F64:
         audiomixer_orc_add_f64 ((gpointer) (outmap.data + out_offset * bpf),
             (gpointer) (inmap.data + in_offset * bpf),
-            num_frames * aagg->info.channels);
+            num_frames * srcpad->info.channels);
         break;
       default:
         g_assert_not_reached ();
         break;
     }
   } else {
-    switch (aagg->info.finfo->format) {
+    switch (srcpad->info.finfo->format) {
       case GST_AUDIO_FORMAT_U8:
         audiomixer_orc_add_volume_u8 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i8, num_frames * aagg->info.channels);
+            pad->volume_i8, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S8:
         audiomixer_orc_add_volume_s8 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i8, num_frames * aagg->info.channels);
+            pad->volume_i8, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_U16:
         audiomixer_orc_add_volume_u16 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i16, num_frames * aagg->info.channels);
+            pad->volume_i16, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S16:
         audiomixer_orc_add_volume_s16 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i16, num_frames * aagg->info.channels);
+            pad->volume_i16, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_U32:
         audiomixer_orc_add_volume_u32 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i32, num_frames * aagg->info.channels);
+            pad->volume_i32, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_S32:
         audiomixer_orc_add_volume_s32 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume_i32, num_frames * aagg->info.channels);
+            pad->volume_i32, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_F32:
         audiomixer_orc_add_volume_f32 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume, num_frames * aagg->info.channels);
+            pad->volume, num_frames * srcpad->info.channels);
         break;
       case GST_AUDIO_FORMAT_F64:
         audiomixer_orc_add_volume_f64 ((gpointer) (outmap.data +
                 out_offset * bpf), (gpointer) (inmap.data + in_offset * bpf),
-            pad->volume, num_frames * aagg->info.channels);
+            pad->volume, num_frames * srcpad->info.channels);
         break;
       default:
         g_assert_not_reached ();
@@ -453,7 +457,7 @@ gst_audiomixer_child_proxy_init (gpointer g_iface, gpointer iface_data)
 {
   GstChildProxyInterface *iface = g_iface;
 
-  GST_INFO ("intializing child proxy interface");
+  GST_INFO ("initializing child proxy interface");
   iface->get_child_by_index = gst_audiomixer_child_proxy_get_child_by_index;
   iface->get_children_count = gst_audiomixer_child_proxy_get_children_count;
 }
