@@ -169,7 +169,7 @@ struct _OutputPad
   /* Downstream event probe id */
   gulong probe_id;
 
-  /* TRUE if the pad saw EOS. Resetted to FALSE on STREAM_START */
+  /* TRUE if the pad saw EOS. Reset to FALSE on STREAM_START */
   gboolean is_eos;
 
   /* The last seen (i.e. current) group_id
@@ -236,6 +236,15 @@ struct _GstURIDecodeBin3
   gboolean posted_about_to_finish;
 };
 
+static gint
+gst_uridecodebin3_select_stream (GstURIDecodeBin3 * dbin,
+    GstStreamCollection * collection, GstStream * stream)
+{
+  GST_LOG_OBJECT (dbin, "default select-stream, returning -1");
+
+  return -1;
+}
+
 struct _GstURIDecodeBin3Class
 {
   GstBinClass parent_class;
@@ -264,10 +273,8 @@ static GstStaticCaps raw_video_caps = GST_STATIC_CAPS ("video/x-raw(ANY)");
 /* properties */
 #define DEFAULT_PROP_URI            NULL
 #define DEFAULT_PROP_SUBURI            NULL
-#define DEFAULT_PROP_SOURCE         NULL
 #define DEFAULT_CONNECTION_SPEED    0
 #define DEFAULT_CAPS                (gst_static_caps_get (&default_raw_caps))
-#define DEFAULT_SUBTITLE_ENCODING   NULL
 #define DEFAULT_BUFFER_DURATION     -1
 #define DEFAULT_BUFFER_SIZE         -1
 #define DEFAULT_DOWNLOAD            FALSE
@@ -282,7 +289,6 @@ enum
   PROP_SUBURI,
   PROP_CURRENT_SUBURI,
   PROP_SOURCE,
-  PROP_SUBTITLE_ENCODING,
   PROP_CONNECTION_SPEED,
   PROP_BUFFER_SIZE,
   PROP_BUFFER_DURATION,
@@ -334,6 +340,8 @@ static void gst_uri_decode_bin3_set_property (GObject * object, guint prop_id,
 static void gst_uri_decode_bin3_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_uri_decode_bin3_finalize (GObject * obj);
+static GstSourceHandler *new_source_handler (GstURIDecodeBin3 * uridecodebin,
+    gboolean is_main);
 
 static GstStateChangeReturn gst_uri_decode_bin3_change_state (GstElement *
     element, GstStateChange transition);
@@ -389,14 +397,6 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
       g_param_spec_object ("source", "Source", "Source object used",
           GST_TYPE_ELEMENT, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_SUBTITLE_ENCODING,
-      g_param_spec_string ("subtitle-encoding", "subtitle encoding",
-          "Encoding to assume if input subtitles are not in UTF-8 encoding. "
-          "If not set, the GST_SUBTITLE_ENCODING environment variable will "
-          "be checked for an encoding to use. If that is not set either, "
-          "ISO-8859-15 will be assumed.", NULL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   g_object_class_install_property (gobject_class, PROP_CONNECTION_SPEED,
       g_param_spec_uint64 ("connection-speed", "Connection Speed",
           "Network connection speed in kbps (0 = unknown)",
@@ -465,14 +465,18 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
    * This signal is emitted whenever @decodebin needs to decide whether
    * to expose a @stream of a given @collection.
    *
+   * Note that the prefered way to select streams is to listen to
+   * GST_MESSAGE_STREAM_COLLECTION on the bus and send a
+   * GST_EVENT_SELECT_STREAMS with the streams the user wants.
+   *
    * Returns: 1 if the stream should be selected, 0 if it shouldn't be selected.
    * A value of -1 (default) lets @decodebin decide what to do with the stream.
    * */
   gst_uri_decode_bin3_signals[SIGNAL_SELECT_STREAM] =
       g_signal_new ("select-stream", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstURIDecodeBin3Class, select_stream),
-      _gst_int_accumulator, NULL, g_cclosure_marshal_generic,
-      G_TYPE_INT, 2, GST_TYPE_STREAM_COLLECTION, GST_TYPE_STREAM);
+      _gst_int_accumulator, NULL, NULL, G_TYPE_INT, 2,
+      GST_TYPE_STREAM_COLLECTION, GST_TYPE_STREAM);
 
   /**
    * GstURIDecodeBin3::source-setup:
@@ -486,18 +490,16 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
    */
   gst_uri_decode_bin3_signals[SIGNAL_SOURCE_SETUP] =
       g_signal_new ("source-setup", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
   /**
    * GstURIDecodeBin3::about-to-finish:
    *
    * This signal is emitted when the data for the selected URI is
-   * entirely buffered and it is safe to specify anothe URI.
+   * entirely buffered and it is safe to specify another URI.
    */
   gst_uri_decode_bin3_signals[SIGNAL_ABOUT_TO_FINISH] =
       g_signal_new ("about-to-finish", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE,
-      0, G_TYPE_NONE);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0, G_TYPE_NONE);
 
 
   gst_element_class_add_static_pad_template (gstelement_class,
@@ -514,6 +516,7 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
 
   gstelement_class->change_state = gst_uri_decode_bin3_change_state;
 
+  klass->select_stream = gst_uridecodebin3_select_stream;
 }
 
 static GstPadProbeReturn
@@ -575,6 +578,9 @@ db_pad_removed_cb (GstElement * element, GstPad * pad, GstURIDecodeBin3 * dec)
   GList *tmp;
   OutputPad *output = NULL;
 
+  if (!GST_PAD_IS_SRC (pad))
+    return;
+
   GST_DEBUG_OBJECT (dec, "pad %s:%s", GST_DEBUG_PAD_NAME (pad));
   /* FIXME: LOCK for list access */
 
@@ -583,7 +589,7 @@ db_pad_removed_cb (GstElement * element, GstPad * pad, GstURIDecodeBin3 * dec)
 
     if (cand->target_pad == pad) {
       output = cand;
-      dec->output_pads = g_list_remove_link (dec->output_pads, tmp);
+      dec->output_pads = g_list_delete_link (dec->output_pads, tmp);
       break;
     }
   }
@@ -631,7 +637,15 @@ gst_uri_decode_bin3_init (GstURIDecodeBin3 * dec)
 {
   g_mutex_init (&dec->lock);
 
-  dec->caps = gst_static_caps_get (&default_raw_caps);
+  dec->uri = DEFAULT_PROP_URI;
+  dec->suburi = DEFAULT_PROP_SUBURI;
+  dec->connection_speed = DEFAULT_CONNECTION_SPEED;
+  dec->caps = DEFAULT_CAPS;
+  dec->buffer_duration = DEFAULT_BUFFER_DURATION;
+  dec->buffer_size = DEFAULT_BUFFER_SIZE;
+  dec->download = DEFAULT_DOWNLOAD;
+  dec->use_buffering = DEFAULT_USE_BUFFERING;
+  dec->ring_buffer_max_size = DEFAULT_RING_BUFFER_MAX_SIZE;
 
   dec->decodebin = gst_element_factory_make ("decodebin3", NULL);
   gst_bin_add (GST_BIN_CAST (dec), dec->decodebin);
@@ -659,6 +673,8 @@ gst_uri_decode_bin3_finalize (GObject * obj)
   GstURIDecodeBin3 *dec = GST_URI_DECODE_BIN3 (obj);
 
   g_mutex_clear (&dec->lock);
+  g_free (dec->uri);
+  g_free (dec->suburi);
 
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -691,8 +707,11 @@ src_pad_added_cb (GstElement * element, GstPad * pad,
   GstURIDecodeBin3 *uridecodebin;
   GstPad *sinkpad = NULL;
   GstPadLinkReturn res;
+  GstPlayItem *current_play_item;
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   uridecodebin = handler->uridecodebin;
+  current_play_item = uridecodebin->current;
 
   GST_DEBUG_OBJECT (uridecodebin,
       "New pad %" GST_PTR_FORMAT " from source %" GST_PTR_FORMAT, pad, element);
@@ -719,6 +738,17 @@ src_pad_added_cb (GstElement * element, GstPad * pad,
     if (GST_PAD_LINK_FAILED (res))
       goto link_failed;
   }
+
+  /* Activate sub_item after the main source activation was finished */
+  if (handler->is_main_source && current_play_item->sub_item
+      && !current_play_item->sub_item->handler) {
+    current_play_item->sub_item->handler =
+        new_source_handler (uridecodebin, FALSE);
+    ret = activate_source_item (current_play_item->sub_item);
+    if (ret == GST_STATE_CHANGE_FAILURE)
+      goto sub_item_activation_failed;
+  }
+
   return;
 
 link_failed:
@@ -728,6 +758,12 @@ link_failed:
         GST_DEBUG_PAD_NAME (pad), gst_pad_link_get_name (res), res);
     return;
   }
+sub_item_activation_failed:
+  {
+    GST_ERROR_OBJECT (uridecodebin,
+        "failed to activate subtitle playback item");
+    return;
+  }
 }
 
 static void
@@ -811,10 +847,6 @@ gst_uri_decode_bin3_set_property (GObject * object, guint prop_id,
         g_free (dec->suburi);
       dec->suburi = g_value_dup_string (value);
       break;
-    case PROP_SUBTITLE_ENCODING:
-      g_object_set_property (G_OBJECT (dec->decodebin), "subtitle-encoding",
-          value);
-      break;
     case PROP_CONNECTION_SPEED:
       GST_URI_DECODE_BIN3_LOCK (dec);
       dec->connection_speed = g_value_get_uint64 (value) * 1000;
@@ -862,19 +894,25 @@ gst_uri_decode_bin3_get_property (GObject * object, guint prop_id,
     }
     case PROP_CURRENT_URI:
     {
-      g_value_set_string (value, dec->suburi);
+      if (dec->current && dec->current->main_item) {
+        g_value_set_string (value, dec->current->main_item->uri);
+      } else {
+        g_value_set_string (value, NULL);
+      }
       break;
     }
     case PROP_SUBURI:
     {
-      /* FIXME : Return current uri */
-      g_value_set_string (value, dec->uri);
+      g_value_set_string (value, dec->suburi);
       break;
     }
     case PROP_CURRENT_SUBURI:
     {
-      /* FIXME : Return current suburi */
-      g_value_set_string (value, dec->suburi);
+      if (dec->current && dec->current->sub_item) {
+        g_value_set_string (value, dec->current->sub_item->uri);
+      } else {
+        g_value_set_string (value, NULL);
+      }
       break;
     }
     case PROP_SOURCE:
@@ -884,10 +922,6 @@ gst_uri_decode_bin3_get_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (dec);
       break;
     }
-    case PROP_SUBTITLE_ENCODING:
-      g_object_get_property (G_OBJECT (dec->decodebin), "subtitle-encoding",
-          value);
-      break;
     case PROP_CONNECTION_SPEED:
       GST_URI_DECODE_BIN3_LOCK (dec);
       g_value_set_uint64 (value, dec->connection_speed / 1000);
@@ -1002,11 +1036,6 @@ assign_handlers_to_item (GstURIDecodeBin3 * dec, GstPlayItem * item)
       return ret;
   }
 
-  if (item->sub_item && item->sub_item->handler) {
-    item->sub_item->handler = new_source_handler (dec, FALSE);
-    ret = activate_source_item (item->sub_item);
-  }
-
   return ret;
 }
 
@@ -1028,6 +1057,7 @@ activate_next_play_item (GstURIDecodeBin3 * dec)
   }
 
   dec->play_items = g_list_append (dec->play_items, item);
+  dec->current = dec->play_items->data;
 
   return ret;
 }
@@ -1082,6 +1112,8 @@ gst_uri_decode_bin3_change_state (GstElement * element,
   /* ERRORS */
 failure:
   {
+    if (transition == GST_STATE_CHANGE_READY_TO_PAUSED)
+      free_play_items (uridecodebin);
     return ret;
   }
 }
