@@ -20,17 +20,18 @@
 
 /**
  * SECTION:element-tcpserversink
+ * @title: tcpserversink
  * @see_also: #multifdsink
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line (server):
  * |[
- * # server:
  * gst-launch-1.0 fdsrc fd=1 ! tcpserversink port=3000
- * # client:
+ * ]|
+ * ## Example launch line (client):
+ * |[
  * gst-launch-1.0 tcpclientsrc port=3000 ! fdsink fd=2
- * ]| 
- * </refsect2>
+ * ]|
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -245,7 +246,7 @@ gst_tcp_server_sink_set_property (GObject * object, guint prop_id,
         break;
       }
       g_free (sink->host);
-      sink->host = g_strdup (g_value_get_string (value));
+      sink->host = g_value_dup_string (value);
       break;
     case PROP_PORT:
       sink->server_port = g_value_get_int (value);
